/*! Tripetto Runner Foundation 3.3.0 - Copyright (C) 2020 Tripetto B.V. - All Rights Reserved */

declare module "@marblecore/callback" {
    /** Explicit exports */
    export { Await } from "@marblecore/callback/lib/await";
    export {
        Callback,
        ICallbackProperties,
        ICallbackPromise,
    } from "@marblecore/callback/lib/callback";
    export {
        IAwaitProperties,
        IAwaitPromise,
    } from "@marblecore/callback/lib/await";
}

declare module "@marblecore/callback/lib/await" {
    /** Dependencies */
    import {
        Callback,
        ICallbackPromise,
        TPromise,
    } from "@marblecore/callback/lib/callback";
    /**
     * Type alias for the await function.
     * @param TAwait.pAwait Reference to the await.
     */
    export type TAwait = (pAwait: Await) => void | Await;
    /** Defines the promise interface. */
    export interface IAwaitPromise extends ICallbackPromise<void> {}
    /** Defines the callback properties interface. */
    export interface IAwaitProperties {
        /** Specifies the await function. */
        readonly await: TAwait;
        /** Specifies the promise function. */
        readonly promise: TPromise<void>;
        /**
         * Specifies if the callback is conditional. In this case the callback
         * condition must be set to true before a callback is accepted. The callback
         * condition can only be used for asynchronous callbacks.
         */
        readonly conditional?: boolean;
        /**
         * Specifies if the callback should be continuous. The callback function
         * `done()` can be called multiple times.
         */
        readonly continuous?: boolean;
    }
    export class Await extends Callback<void> {
        /**
         * Await for.
         * @param pProperties Specifies the await properties.
         * @return Returns a reference to the await.
         */
        static for(pProperties: IAwaitProperties): Await;
        /**
         * Creates a new await.
         * @param pProperties Specifies the await properties.
         */
        constructor(pProperties: IAwaitProperties);
        /**
         * Terminates the await.
         * @return Returns `true` if the await loop is terminated.
         */
        done(): boolean;
    }
}

declare module "@marblecore/callback/lib/callback" {
    /**
     * Type alias for the await function.
     * @param TAwait.pCallback Reference to the callback.
     */
    export type TAwait<T> = (pCallback: Callback<T>) => T | Callback<T>;
    /**
     * Type alias for the promise function.
     * @param TPromise.pPromise Contains the promise details.
     */
    export type TPromise<T> =
        | ((pPromise: ICallbackPromise<T>) => void)
        | undefined;
    /** Defines the promise interface. */
    export interface ICallbackPromise<T> {
        /** Specifies if the await was asynchronous. */
        readonly isAsynchronous: boolean;
        /** Specifies if the await was canceled. */
        readonly isCanceled: boolean;
        /** Specifies the payload which is returned. */
        readonly payload: T | undefined;
    }
    /** Defines the callback properties interface. */
    export interface ICallbackProperties<T> {
        /** Specifies the await function. */
        readonly await: TAwait<T>;
        /** Specifies the promise function. */
        readonly promise: TPromise<T>;
        /**
         * Specifies if the callback is conditional. In this case the callback
         * condition must be set to true before a callback is accepted. The callback
         * condition can only be used for asynchronous callbacks.
         */
        readonly conditional?: boolean;
        /**
         * Specifies if the callback should be continuous. The callback property
         * `Payload` or function `done()` can be called multiple times.
         */
        readonly continuous?: boolean;
    }
    export class Callback<T> {
        /**
         * Callback template of specified type.
         * @param pProperties Specifies the callback properties.
         * @return Returns a reference to the callback.
         */
        static of<Type>(pProperties: ICallbackProperties<Type>): Callback<Type>;
        /**
         * Creates a new callback.
         * @param pProperties Specifies the callback properties.
         */
        constructor(pProperties: ICallbackProperties<T>);
        /** Retrieves the callback condition state. */
        get condition(): boolean;
        /** Sets the callback condition state. */
        set condition(bCondition: boolean);
        /** Verifies if the callback pointer is alive and still usable. */
        get isAlive(): boolean;
        /** Retrieves the promise invoke count. */
        get promiseCount(): number;
        /**
         * Terminates the callback.
         * @return Returns `true` if the callback loop is terminated.
         */
        protected terminate(): boolean;
        /**
         * Cancels the callback loop.
         * @return Returns `true` if the callback loop is terminated.
         */
        cancel(): boolean;
        /** Sets the return payload of the callback function. */
        return(pPayload: T | undefined): T | undefined;
    }
}
